@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011, 2012, 2015 Red Hat, Inc.
+ * Copyright (C) 2011, 2012, 2015, 2016 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -429,7 +429,7 @@ typedef struct
 static void
 key_file_data_free (KeyFileData *data)
 {
-  /* the key_file member is freed elsewhere */
+  g_key_file_unref (data->key_file);
   g_free (data->path);
   g_slice_free (KeyFileData, data);
 }
@@ -440,7 +440,7 @@ key_file_data_new (GKeyFile    *key_file,
 {
   KeyFileData *data;
   data = g_slice_new (KeyFileData);
-  data->key_file = key_file;
+  data->key_file = g_key_file_ref (key_file);
   data->path = g_strdup (path);
   return data;
 }
@@ -450,11 +450,15 @@ key_file_data_new (GKeyFile    *key_file,
 static void
 add_config_file (GoaDaemon     *self,
                  const gchar   *path,
-                 GHashTable    *group_name_to_key_file_data,
-                 GList        **key_files_to_free)
+                 GHashTable    *group_name_to_key_file_data)
 {
   GKeyFile *key_file;
   GError *error;
+  gboolean needs_update = FALSE;
+  gchar **groups;
+  const char *guid;
+  gsize num_groups;
+  guint n;
 
   key_file = g_key_file_new ();
 
@@ -471,114 +475,107 @@ add_config_file (GoaDaemon     *self,
                      error->message, g_quark_to_string (error->domain), error->code);
         }
       g_error_free (error);
-      g_key_file_free (key_file);
+      goto out;
     }
-  else
+
+  guid = g_dbus_connection_get_guid (self->connection);
+  groups = g_key_file_get_groups (key_file, &num_groups);
+  for (n = 0; n < num_groups; n++)
     {
-      gboolean needs_update = FALSE;
-      gchar **groups;
-      const char *guid;
-      gsize num_groups;
-      guint n;
-
-      guid = g_dbus_connection_get_guid (self->connection);
-      groups = g_key_file_get_groups (key_file, &num_groups);
-      for (n = 0; n < num_groups; n++)
+      if (g_str_has_prefix (groups[n], "Account "))
         {
-          if (g_str_has_prefix (groups[n], "Account "))
+          gboolean is_temporary;
+          char *session_id;
+
+          is_temporary = g_key_file_get_boolean (key_file,
+                                                 groups[n],
+                                                 "IsTemporary",
+                                                 NULL);
+
+          if (is_temporary)
             {
-              gboolean is_temporary;
-              char *session_id;
+              session_id = g_key_file_get_string (key_file,
+                                                  groups[n],
+                                                  "SessionId",
+                                                  NULL);
 
-              is_temporary = g_key_file_get_boolean (key_file,
-                                                     groups[n],
-                                                     "IsTemporary",
-                                                     NULL);
-
-              if (is_temporary)
+              /* discard temporary accounts from older sessions */
+              if (session_id != NULL &&
+                  g_strcmp0 (session_id, guid) != 0)
                 {
-                  session_id = g_key_file_get_string (key_file,
-                                                      groups[n],
-                                                      "SessionId",
-                                                      NULL);
+                  GoaProvider *provider = NULL;
+                  const gchar *id;
+                  gchar *provider_type = NULL;
 
-                  /* discard temporary accounts from older sessions */
-                  if (session_id != NULL &&
-                      g_strcmp0 (session_id, guid) != 0)
+                  g_debug ("ignoring account \"%s\" in file %s because it's stale",
+                           groups[n], path);
+
+                  id = group_to_id (groups[n]);
+                  if (id == NULL)
                     {
-                      GoaProvider *provider = NULL;
-                      const gchar *id;
-                      gchar *provider_type = NULL;
-
-                      g_debug ("ignoring account \"%s\" in file %s because it's stale",
-                               groups[n], path);
-
-                      id = group_to_id (groups[n]);
-                      if (id == NULL)
-                        {
-                          g_warning ("Unable to get account ID from group: %s", groups[n]);
-                          goto cleanup_and_continue;
-                        }
-
-                      provider_type = g_key_file_get_string (key_file, groups[n], "Provider", NULL);
-                      if (provider_type != NULL)
-                        provider = goa_provider_get_for_provider_type (provider_type);
-
-                      if (provider == NULL)
-                        {
-                          g_warning ("Unsupported account type %s for ID %s (no provider)", provider_type, id);
-                          goto cleanup_and_continue;
-                        }
-
-                      needs_update = g_key_file_remove_group (key_file, groups[n], NULL);
-
-                      error = NULL;
-                      if (!goa_utils_delete_credentials_for_id_sync (provider, id, NULL, &error))
-                        {
-                          g_warning ("Unable to clean-up stale keyring entries: %s", error->message);
-                          g_error_free (error);
-                          goto cleanup_and_continue;
-                        }
-
-                    cleanup_and_continue:
-                      g_clear_object (&provider);
-                      g_free (groups[n]);
-                      g_free (provider_type);
-                      g_free (session_id);
-                      continue;
+                      g_warning ("Unable to get account ID from group: %s", groups[n]);
+                      goto cleanup_and_continue;
                     }
-                  g_free (session_id);
-                }
-              else
-                {
-                  needs_update = g_key_file_remove_key (key_file, groups[n], "SessionId", NULL);
-                }
 
-              g_hash_table_insert (group_name_to_key_file_data,
-                                   groups[n], /* steals string */
-                                   key_file_data_new (key_file, path));
+                  provider_type = g_key_file_get_string (key_file, groups[n], "Provider", NULL);
+                  if (provider_type != NULL)
+                    provider = goa_provider_get_for_provider_type (provider_type);
+
+                  if (provider == NULL)
+                    {
+                      g_warning ("Unsupported account type %s for ID %s (no provider)", provider_type, id);
+                      goto cleanup_and_continue;
+                    }
+
+                  needs_update = g_key_file_remove_group (key_file, groups[n], NULL);
+
+                  error = NULL;
+                  if (!goa_utils_delete_credentials_for_id_sync (provider, id, NULL, &error))
+                    {
+                      g_warning ("Unable to clean-up stale keyring entries: %s", error->message);
+                      g_error_free (error);
+                      goto cleanup_and_continue;
+                    }
+
+                cleanup_and_continue:
+                  g_clear_object (&provider);
+                  g_free (groups[n]);
+                  g_free (provider_type);
+                  g_free (session_id);
+                  continue;
+                }
+              g_free (session_id);
             }
           else
             {
-              g_warning ("Unexpected group \"%s\" in file %s", groups[n], path);
-              g_free (groups[n]);
+              needs_update = g_key_file_remove_key (key_file, groups[n], "SessionId", NULL);
             }
-        }
-      g_free (groups);
 
-      if (needs_update)
+          g_hash_table_insert (group_name_to_key_file_data,
+                               groups[n], /* steals string */
+                               key_file_data_new (key_file, path));
+        }
+      else
         {
-          error = NULL;
-          if (!g_key_file_save_to_file (key_file, path, &error))
-            {
-              g_prefix_error (&error, "Error writing key-value-file %s: ", path);
-              g_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
-              g_error_free (error);
-            }
+          g_warning ("Unexpected group \"%s\" in file %s", groups[n], path);
+          g_free (groups[n]);
         }
-
-      *key_files_to_free = g_list_prepend (*key_files_to_free, key_file);
     }
+  g_free (groups);
+
+  if (needs_update)
+    {
+      error = NULL;
+      if (!g_key_file_save_to_file (key_file, path, &error))
+        {
+          g_prefix_error (&error, "Error writing key-value-file %s: ", path);
+          g_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
+          g_error_free (error);
+        }
+    }
+
+ out:
+  g_key_file_unref (key_file);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -592,17 +589,17 @@ update_account_object (GoaDaemon           *self,
                        GKeyFile            *key_file,
                        gboolean             just_added)
 {
-  GoaAccount *account;
-  GoaProvider *provider;
+  GoaAccount *account = NULL;
+  GoaProvider *provider = NULL;
   gboolean is_locked;
   gboolean is_temporary;
-  gboolean ret;
-  gchar *identity;
+  gboolean ret = FALSE;
+  gchar *identity = NULL;
   gchar *presentation_identity;
-  gchar *type;
-  gchar *name;
-  GIcon *icon;
-  gchar *serialized_icon;
+  gchar *type = NULL;
+  gchar *name = NULL;
+  GIcon *icon = NULL;
+  gchar *serialized_icon = NULL;
   GError *error;
 
   g_return_val_if_fail (GOA_IS_DAEMON (self), FALSE);
@@ -610,16 +607,6 @@ update_account_object (GoaDaemon           *self,
   g_return_val_if_fail (group != NULL, FALSE);
   g_return_val_if_fail (key_file != NULL, FALSE);
 
-  ret = FALSE;
-  account = NULL;
-  provider = NULL;
-  identity = NULL;
-  type = NULL;
-  account = NULL;
-  name = NULL;
-  icon = NULL;
-  serialized_icon = NULL;
-
   g_debug ("updating %s %d", g_dbus_object_get_object_path (G_DBUS_OBJECT (object)), just_added);
 
   type = g_key_file_get_string (key_file, group, "Provider", NULL);
@@ -688,14 +675,13 @@ process_config_entries (GoaDaemon  *self,
   GHashTableIter iter;
   const gchar *group;
   KeyFileData *key_file_data;
-  GList *existing_object_paths;
-  GList *config_object_paths;
+  GList *existing_object_paths = NULL;
+  GList *config_object_paths = NULL;
   GList *added;
   GList *removed;
   GList *unchanged;
   GList *l;
 
-  existing_object_paths = NULL;
   {
     GList *existing_objects;
     existing_objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (self->object_manager));
@@ -710,7 +696,6 @@ process_config_entries (GoaDaemon  *self,
     g_list_free_full (existing_objects, g_object_unref);
   }
 
-  config_object_paths = NULL;
   g_hash_table_iter_init (&iter, group_name_to_key_file_data);
   while (g_hash_table_iter_next (&iter, (gpointer*) &group, (gpointer*) &key_file_data))
     {
@@ -837,11 +822,9 @@ process_config_entries (GoaDaemon  *self,
 static void
 goa_daemon_reload_configuration (GoaDaemon *self)
 {
-  GList *key_files_to_free;
   GHashTable *group_name_to_key_file_data;
   gchar *path;
 
-  key_files_to_free = NULL;
   group_name_to_key_file_data = g_hash_table_new_full (g_str_hash,
                                                        g_str_equal,
                                                        g_free,
@@ -849,14 +832,13 @@ goa_daemon_reload_configuration (GoaDaemon *self)
 
   /* Read the main user config file at $HOME/.config/goa-1.0/accounts.conf */
   path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  add_config_file (self, path, group_name_to_key_file_data, &key_files_to_free);
+  add_config_file (self, path, group_name_to_key_file_data);
   g_free (path);
 
   /* now process the group_name_to_key_file_data hash table */
   process_config_entries (self, group_name_to_key_file_data);
 
   g_hash_table_unref (group_name_to_key_file_data);
-  g_list_free_full (key_files_to_free, (GDestroyNotify) g_key_file_free);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -896,33 +878,24 @@ get_all_providers_cb (GObject      *source,
                       gpointer      user_data)
 {
   AddAccountData *data = user_data;
-  GoaProvider *provider;
-  GKeyFile *key_file;
+  GoaProvider *provider = NULL;
+  GKeyFile *key_file = NULL;
   GError *error;
-  GList *providers;
+  GList *providers = NULL;
   GList *l;
-  gchar *path;
-  gchar *id;
-  gchar *group;
-  gchar *key_file_data;
+  gchar *path = NULL;
+  gchar *id = NULL;
+  gchar *group = NULL;
+  gchar *key_file_data = NULL;
   gsize length;
   gsize n_credentials;
-  gchar *object_path;
+  gchar *object_path = NULL;
   GVariantIter iter;
   const gchar *key;
   const gchar *value;
 
   /* TODO: could check for @type */
 
-  provider = NULL;
-  key_file = NULL;
-  providers = NULL;
-  path = NULL;
-  id = NULL;
-  group = NULL;
-  key_file_data = NULL;
-  object_path = NULL;
-
   if (!goa_provider_get_all_finish (&providers, res, NULL))
     goto out;
 
@@ -1051,7 +1024,7 @@ get_all_providers_cb (GObject      *source,
   g_free (group);
   g_free (id);
   g_free (path);
-  g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_free);
+  g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_unref);
   g_object_unref (data->daemon);
   g_object_unref (data->manager);
   g_object_unref (data->invocation);
@@ -1160,22 +1133,15 @@ on_account_handle_remove (GoaAccount            *account,
 {
   GoaDaemon *self = GOA_DAEMON (user_data);
   GoaObject *object;
-  GoaProvider *provider;
-  GKeyFile *key_file;
-  GTask *task;
+  GoaProvider *provider = NULL;
+  GKeyFile *key_file = NULL;
+  GTask *task = NULL;
   ObjectInvocationData *data;
-  const gchar *provider_type;
-  gchar *path;
-  gchar *group;
+  const gchar *provider_type = NULL;
+  gchar *path = NULL;
+  gchar *group = NULL;
   GError *error;
 
-  provider = NULL;
-  provider_type = NULL;
-  path = NULL;
-  group = NULL;
-  key_file = NULL;
-  task = NULL;
-
   if (goa_account_get_is_locked (account))
     {
       error = NULL;
@@ -1264,7 +1230,7 @@ on_account_handle_remove (GoaAccount            *account,
  out:
   g_clear_object (&provider);
   g_clear_object (&task);
-  g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_free);
+  g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_unref);
   g_free (group);
   g_free (path);
   return TRUE; /* invocation was handled */
